@@ -1,4 +1,4 @@
-#$Id: Neo4p.pm 291 2013-11-22 02:17:32Z maj $
+#$Id: Neo4p.pm 295 2013-11-27 23:49:11Z maj $
 use v5.10;
 package REST::Neo4p;
 use Carp qw(croak carp);
@@ -14,7 +14,7 @@ use strict;
 use warnings;
 
 BEGIN {
-  $REST::Neo4p::VERSION = '0.2201';
+  $REST::Neo4p::VERSION = '0.2220';
 }
 
 our $CREATE_AUTO_ACCESSORS = 0;
@@ -36,6 +36,7 @@ sub create_and_set_handle {
   my $class = shift;
   $HANDLE = @HANDLES;
   $HANDLES[$HANDLE]->{_agent} = REST::Neo4p::Agent->new;
+  $HANDLES[$HANDLE]->{_q_endpoint} = 'cypher';
   return $HANDLE;
 }
 
@@ -69,6 +70,19 @@ sub _clear_transaction {
   delete $HANDLES[$HANDLE]->{_transaction}
 }
 
+sub _set_autocommit {
+  my $class = shift;
+  return $HANDLES[$HANDLE]->{_q_endpoint} = 'cypher';
+}
+
+sub _clear_autocommit {
+  my $class = shift;
+  if ($class->_check_version(2,0,0,2)) {
+    return $HANDLES[$HANDLE]->{_q_endpoint} = 'transaction';
+  }
+  return;
+}
+
 sub q_endpoint { 
   my $neo4p = shift;
   return $HANDLES[$HANDLE]->{_q_endpoint};
